@@ -281,6 +281,11 @@ public:
     QPointF axesCenter;
     bool allowMasksOnRootNode = false;
 
+    void requestProjectionUpdateImpl(KisNode *node,
+                                     const QVector<QRect> &rects,
+                                     const QRect &cropRect,
+                                     KisProjectionUpdateFlags flags);
+
     bool tryCancelCurrentStrokeAsync();
 
     void notifyProjectionUpdatedInPatches(const QRect &rc, QVector<KisRunnableStrokeJobData *> &jobs);
@@ -405,7 +410,7 @@ void KisImage::copyFromImageImpl(const KisImage &rhs, int policy)
     /**
      * We should replace the root before emitting any signals, because some of the layers
      * may be subscribed to sigSizeChanged() signal (e.g. KisSelectionBasedLayer). So the
-     * old layers should be fully detached before we actually emit this signal.
+     * old layers should be fully detached before we actually Q_EMIT this signal.
      *
      * See bug 447599 for more details.
      */
@@ -425,18 +430,18 @@ void KisImage::copyFromImageImpl(const KisImage &rhs, int policy)
     }
 
 
-    // only when replacing do we need to emit signals
+    // only when replacing do we need to Q_EMIT signals
 #define EMIT_IF_NEEDED if (!(policy & REPLACE)) {} else emit
 
     if (policy & REPLACE) { // if we are constructing the image, these are already set
         if (m_d->width != rhs.width() || m_d->height != rhs.height()) {
             m_d->width = rhs.width();
             m_d->height = rhs.height();
-            emit sigSizeChanged(QPointF(), QPointF());
+            Q_EMIT sigSizeChanged(QPointF(), QPointF());
         }
         if (m_d->colorSpace != rhs.colorSpace()) {
             m_d->colorSpace = rhs.colorSpace();
-            emit sigColorSpaceChanged(m_d->colorSpace);
+            Q_EMIT sigColorSpaceChanged(m_d->colorSpace);
         }
     }
 
@@ -579,7 +584,7 @@ void KisImage::aboutToRemoveANode(KisNode *parent, int index)
     if (!dynamic_cast<KisSelectionMask*>(deletedNode.data()) &&
         deletedNode == m_d->isolationRootNode) {
 
-        emit sigInternalStopIsolatedModeRequested();
+        Q_EMIT sigInternalStopIsolatedModeRequested();
     }
 
     KisLayerUtils::recursiveApplyNodes(KisSharedPtr<KisNode>(parent), [this](KisNodeSP node){
@@ -1488,8 +1493,17 @@ double KisImage::yRes() const
 
 void KisImage::setResolution(double xres, double yres)
 {
-    m_d->xres = xres;
-    m_d->yres = yres;
+    if (xres > 0) {
+        m_d->xres = xres;
+    } else {
+        qWarning() << "WARNING: Ignoring attempt to set image x resolution <= 0 (" << xres << ")!";
+    }
+
+    if (yres > 0) {
+        m_d->yres = yres;
+    } else {
+        qWarning() << "WARNING: Ignoring attempt to set image y resolution <= 0 (" << yres << ")!";
+    }
 }
 
 QPointF KisImage::documentToPixel(const QPointF &documentCoord) const
@@ -1611,7 +1625,7 @@ void KisImage::flattenLayer(KisLayerSP layer)
 void KisImage::setModifiedWithoutUndo()
 {
     m_d->signalRouter.emitNotification(ModifiedWithoutUndoSignal);
-    emit sigImageModified();
+    Q_EMIT sigImageModified();
 }
 
 QImage KisImage::convertToQImage(QRect imageRect,
@@ -1747,7 +1761,7 @@ KoColor KisImage::defaultProjectionColor() const
 
 void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
 {
-    emit sigInternalStopIsolatedModeRequested();
+    Q_EMIT sigInternalStopIsolatedModeRequested();
 
     KoColor defaultProjectionColor = KoColor::createTransparent(m_d->colorSpace);
 
@@ -1825,7 +1839,7 @@ vKisAnnotationSP_it KisImage::endAnnotations()
 
 void KisImage::notifyAboutToBeDeleted()
 {
-    emit sigAboutToBeDeleted();
+    Q_EMIT sigAboutToBeDeleted();
 }
 
 KisImageSignalRouter* KisImage::signalRouter()
@@ -1914,7 +1928,7 @@ bool KisImage::startIsolatedMode(KisNodeSP node, bool isolateLayer, bool isolate
             const bool prevRootBeforeVisibility = m_prevRoot ? m_prevRoot->projectionLeaf()->visible() : false;
 
             m_image->m_d->isolationRootNode = m_newRoot;
-            emit m_image->sigIsolatedModeChanged();
+            Q_EMIT m_image->sigIsolatedModeChanged();
 
             const bool afterVisibility = m_newRoot->projectionLeaf()->visible();
             const bool prevRootAfterVisibility = m_prevRoot ? m_prevRoot->projectionLeaf()->visible() : false;
@@ -1925,7 +1939,7 @@ bool KisImage::startIsolatedMode(KisNodeSP node, bool isolateLayer, bool isolate
 
         void finishStrokeCallback() override {
             // the GUI uses our thread to do the color space conversion so we
-            // need to emit this signal in multiple threads
+            // need to Q_EMIT this signal in multiple threads
 
             if (m_prevRoot && m_prevRootNeedsFullRefresh) {
                 m_image->refreshGraphAsync(m_prevRoot);
@@ -1987,7 +2001,7 @@ void KisImage::stopIsolatedMode()
             m_image->m_d->isolationRootNode = 0;
             m_image->m_d->isolateLayer = false;
             m_image->m_d->isolateGroup = false;
-            emit m_image->sigIsolatedModeChanged();
+            Q_EMIT m_image->sigIsolatedModeChanged();
             const bool afterVisibility = m_oldRootNode->projectionLeaf()->visible();
 
             m_oldNodeNeedsRefresh = (beforeVisibility != afterVisibility);
@@ -2059,18 +2073,18 @@ bool KisImage::KisImagePrivate::tryCancelCurrentStrokeAsync()
 
 void KisImage::requestUndoDuringStroke()
 {
-    emit sigUndoDuringStrokeRequested();
+    Q_EMIT sigUndoDuringStrokeRequested();
 }
 
 void KisImage::requestRedoDuringStroke()
 {
-    emit sigRedoDuringStrokeRequested();
+    Q_EMIT sigRedoDuringStrokeRequested();
 }
 
 void KisImage::requestStrokeCancellation()
 {
     if (!m_d->tryCancelCurrentStrokeAsync()) {
-        emit sigStrokeCancellationRequested();
+        Q_EMIT sigStrokeCancellationRequested();
     }
 }
 
@@ -2081,26 +2095,13 @@ UndoResult KisImage::tryUndoUnfinishedLod0Stroke()
 
 void KisImage::requestStrokeEnd()
 {
-    emit sigStrokeEndRequested();
-    emit sigStrokeEndRequestedActiveNodeFiltered();
+    Q_EMIT sigStrokeEndRequested();
+    Q_EMIT sigStrokeEndRequestedActiveNodeFiltered();
 }
 
 void KisImage::requestStrokeEndActiveNode()
 {
-    emit sigStrokeEndRequested();
-}
-
-void KisImage::refreshGraph(KisNodeSP root)
-{
-    refreshGraph(root, bounds(), bounds());
-}
-
-void KisImage::refreshGraph(KisNodeSP root, const QRect &rc, const QRect &cropRect)
-{
-    if (!root) root = m_d->rootLayer;
-
-    m_d->animationInterface->notifyNodeChanged(root.data(), rc, true);
-    m_d->scheduler.fullRefresh(root, rc, cropRect);
+    Q_EMIT sigStrokeEndRequested();
 }
 
 void KisImage::initialRefreshGraph()
@@ -2114,64 +2115,42 @@ void KisImage::initialRefreshGraph()
     waitForDone();
 }
 
-void KisImage::refreshGraphAsync(KisNodeSP root, UpdateFlags flags)
-{
-    refreshGraphAsync(root, bounds(), bounds(), flags);
-}
-
-void KisImage::refreshGraphAsync(KisNodeSP root, const QRect &rc, UpdateFlags flags)
-{
-    refreshGraphAsync(root, rc, bounds(), flags);
-}
-
-void KisImage::refreshGraphAsync(KisNodeSP root, const QRect &rc, const QRect &cropRect, UpdateFlags flags)
-{
-    refreshGraphAsync(root, QVector<QRect>({rc}), cropRect, flags);
-}
-
-void KisImage::refreshGraphAsync(KisNodeSP root, const QVector<QRect> &rects, const QRect &cropRect, UpdateFlags flags)
+void KisImage::refreshGraphAsync(KisNodeSP root, const QVector<QRect> &rects, const QRect &cropRect, KisProjectionUpdateFlags flags)
 {
     if (!root) root = m_d->rootLayer;
 
-    /**
-     * We iterate through the filters in a reversed way. It makes the most nested filters
-     * to execute first.
-     */
-    for (auto it = m_d->projectionUpdatesFilters.rbegin();
-         it != m_d->projectionUpdatesFilters.rend();
-         ++it) {
+    QVector<QRect> requestedRects = rects;
 
-        KIS_SAFE_ASSERT_RECOVER(*it) { continue; }
+    KisGroupLayer *group = dynamic_cast<KisGroupLayer*>(root.data());
+    if (group && group->passThroughMode()) {
+        /**
+         * We cannot regenerate a pass-through group, since it is not present in
+         * the layers stack. Instead we should delegate this regeneration pass
+         * to the parent group.
+         *
+         * We should also take into account the change rect of all the layers that lay
+         * above the "dirty" pass-through group (i.e. convert the change rect of the
+         * child into a change rect of the parent). That is required for a case,
+         * when threre are adjustment layers laying above the pass-through group.
+         */
+        QVector<QRect> changeRects = requestedRects;
+        KisProjectionLeafSP leaf = root->projectionLeaf()->nextSibling();
+        while (leaf) {
+            if (!leaf->shouldBeRendered()) continue;
 
-        if ((*it)->filterRefreshGraph(this, root.data(), rects, cropRect, flags)) {
-            return;
+            for (auto it = changeRects.begin(); it != changeRects.end(); ++it) {
+                *it = leaf->projectionPlane()->changeRect(*it, leaf->node() == root ? KisNode::N_FILTHY : KisNode::N_ABOVE_FILTHY);
+            }
+
+            leaf = leaf->nextSibling();
         }
+
+        std::swap(requestedRects, changeRects);
+        root = group->parent();
+
+        KIS_SAFE_ASSERT_RECOVER_RETURN(root);
     }
 
-    m_d->animationInterface->notifyNodeChanged(root.data(), rects, true);
-
-    if (flags & NoFilthyUpdate) {
-        m_d->scheduler.fullRefreshAsyncNoFilthy(root, rects, cropRect);
-    } else {
-        m_d->scheduler.fullRefreshAsync(root, rects, cropRect);
-    }
-}
-
-
-void KisImage::requestProjectionUpdateNoFilthy(KisNodeSP pseudoFilthy, const QRect &rc, const QRect &cropRect)
-{
-    requestProjectionUpdateNoFilthy(pseudoFilthy, rc, cropRect, true);
-}
-
-void KisImage::requestProjectionUpdateNoFilthy(KisNodeSP pseudoFilthy, const QRect &rc, const QRect &cropRect, const bool resetAnimationCache)
-{
-    requestProjectionUpdateNoFilthy(pseudoFilthy, QVector<QRect>({rc}), cropRect, resetAnimationCache);
-}
-
-void KisImage::requestProjectionUpdateNoFilthy(KisNodeSP pseudoFilthy, const QVector<QRect> &rects, const QRect &cropRect, const bool resetAnimationCache)
-{
-    KIS_ASSERT_RECOVER_RETURN(pseudoFilthy);
-
     /**
      * We iterate through the filters in a reversed way. It makes the most nested filters
      * to execute first.
@@ -2182,16 +2161,16 @@ void KisImage::requestProjectionUpdateNoFilthy(KisNodeSP pseudoFilthy, const QVe
 
         KIS_SAFE_ASSERT_RECOVER(*it) { continue; }
 
-        if ((*it)->filterProjectionUpdateNoFilthy(this, pseudoFilthy.data(), rects, cropRect, resetAnimationCache)) {
+        if ((*it)->filterRefreshGraph(this, root.data(), requestedRects, cropRect, flags)) {
             return;
         }
     }
 
-    if (resetAnimationCache) {
-        m_d->animationInterface->notifyNodeChanged(pseudoFilthy.data(), rects, false);
+    if (!flags.testFlag(KisProjectionUpdateFlag::DontInvalidateFrames)) {
+        m_d->animationInterface->notifyNodeChanged(root.data(), requestedRects, true);
     }
 
-    m_d->scheduler.updateProjectionNoFilthy(pseudoFilthy, rects, cropRect);
+    m_d->scheduler.fullRefreshAsync(root, requestedRects, cropRect, flags);
 }
 
 void KisImage::addSpontaneousJob(KisSpontaneousJob *spontaneousJob)
@@ -2291,7 +2270,7 @@ void KisImage::notifyProjectionUpdated(const QRect &rc)
 
         if (dirtyRect.isEmpty()) return;
 
-        emit sigImageUpdated(dirtyRect);
+        Q_EMIT sigImageUpdated(dirtyRect);
     } else {
         m_d->savedDisabledUIUpdates.push(rc);
     }
@@ -2328,16 +2307,18 @@ void KisImage::notifySelectionChanged()
     }
 }
 
-void KisImage::requestProjectionUpdateImpl(KisNode *node,
-                                           const QVector<QRect> &rects,
-                                           const QRect &cropRect)
+void KisImage::KisImagePrivate::
+    requestProjectionUpdateImpl(KisNode *node,
+                                const QVector<QRect> &rects,
+                                const QRect &cropRect,
+                                KisProjectionUpdateFlags flags)
 {
     if (rects.isEmpty()) return;
 
-    m_d->scheduler.updateProjection(node, rects, cropRect);
+    scheduler.updateProjection(node, rects, cropRect, flags);
 }
 
-void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rects, bool resetAnimationCache)
+void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rects, KisProjectionUpdateFlags flags)
 {
     /**
      * We iterate through the filters in a reversed way. It makes the most nested filters
@@ -2349,12 +2330,12 @@ void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rect
 
         KIS_SAFE_ASSERT_RECOVER(*it) { continue; }
 
-        if ((*it)->filter(this, node, rects, resetAnimationCache)) {
+        if ((*it)->filter(this, node, rects, flags)) {
             return;
         }
     }
 
-    if (resetAnimationCache) {
+    if (!flags.testFlag(KisProjectionUpdateFlag::DontInvalidateFrames)) {
         m_d->animationInterface->notifyNodeChanged(node, rects, false);
     }
 
@@ -2364,7 +2345,9 @@ void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rect
      * finished. And having some more updates for the stroke not
      * supporting the wrap-around mode will not make much harm.
      */
-    if (m_d->wrapAroundModePermitted) {
+    if (m_d->wrapAroundModePermitted && !flags.testFlag(KisProjectionUpdateFlag::NoFilthy)) {
+        /// TODO: Remove the no-filthy condition. I have no idea, why it is present.
+        ///       Or replace it with an assert. (DK)
         QVector<QRect> allSplitRects;
 
         const QRect boundRect = effectiveLodBounds();
@@ -2373,13 +2356,13 @@ void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rect
             allSplitRects.append(splitRect);
         }
 
-        requestProjectionUpdateImpl(node, allSplitRects, boundRect);
+        m_d->requestProjectionUpdateImpl(node, allSplitRects, boundRect, flags);
 
     } else {
-        requestProjectionUpdateImpl(node, rects, bounds());
+        m_d->requestProjectionUpdateImpl(node, rects, bounds(), flags);
     }
 
-    KisNodeGraphListener::requestProjectionUpdate(node, rects, resetAnimationCache);
+    KisNodeGraphListener::requestProjectionUpdate(node, rects, flags);
 }
 
 void KisImage::invalidateFrames(const KisTimeSpan &range, const QRect &rect)
@@ -2542,7 +2525,7 @@ KisLodPreferences KisImage::lodPreferences() const
 void KisImage::nodeCollapsedChanged(KisNode * node)
 {
     Q_UNUSED(node);
-    emit sigNodeCollapsedChanged();
+    Q_EMIT sigNodeCollapsedChanged();
 }
 
 KisImageAnimationInterface* KisImage::animationInterface() const
@@ -2553,7 +2536,7 @@ KisImageAnimationInterface* KisImage::animationInterface() const
 void KisImage::setProofingConfiguration(KisProofingConfigurationSP proofingConfig)
 {
     m_d->proofingConfig = proofingConfig;
-    emit sigProofingConfigChanged();
+    Q_EMIT sigProofingConfigChanged();
 }
 
 KisProofingConfigurationSP KisImage::proofingConfiguration() const
