@@ -222,7 +222,7 @@ struct SetKeyStrokesColorSpaceCommand : public KUndo2Command {
         }
 
         m_node->setNeedsUpdate(true);
-        emit m_node->sigKeyStrokesListChanged();
+        Q_EMIT m_node->sigKeyStrokesListChanged();
     }
 
     void redo() override {
@@ -241,7 +241,7 @@ struct SetKeyStrokesColorSpaceCommand : public KUndo2Command {
         }
 
         m_node->setNeedsUpdate(true);
-        emit m_node->sigKeyStrokesListChanged();
+        Q_EMIT m_node->sigKeyStrokesListChanged();
     }
 
 private:
@@ -513,15 +513,17 @@ bool KisColorizeMask::Private::shouldShowColoring() const
 QRect KisColorizeMask::decorateRect(KisPaintDeviceSP &src,
                                     KisPaintDeviceSP &dst,
                                     const QRect &rect,
-                                    PositionToFilthy maskPos) const
+                                    PositionToFilthy maskPos,
+                                    KisRenderPassFlags flags) const
 {
     Q_UNUSED(maskPos);
+    Q_UNUSED(flags);
 
     if (maskPos == N_ABOVE_FILTHY) {
         // the source layer has changed, we should update the filtered cache!
 
         if (!m_d->filteringDirty) {
-            emit sigUpdateOnDirtyParent();
+            Q_EMIT sigUpdateOnDirtyParent();
         }
     }
 
@@ -714,14 +716,14 @@ struct KeyStrokeAddRemoveCommand : public KisCommandUtils::FlipFlopCommand {
     void partA() override {
         m_list->insert(m_index, m_stroke);
         m_node->setNeedsUpdate(true);
-        emit m_node->sigKeyStrokesListChanged();
+        Q_EMIT m_node->sigKeyStrokesListChanged();
     }
 
     void partB() override {
         KIS_ASSERT_RECOVER_RETURN((*m_list)[m_index] == m_stroke);
         m_list->removeAt(m_index);
         m_node->setNeedsUpdate(true);
-        emit m_node->sigKeyStrokesListChanged();
+        Q_EMIT m_node->sigKeyStrokesListChanged();
     }
 
 private:
@@ -906,7 +908,7 @@ struct SetKeyStrokeColorsCommand : public KUndo2Command {
         *m_list = m_newList;
 
         m_node->setNeedsUpdate(true);
-        emit m_node->sigKeyStrokesListChanged();
+        Q_EMIT m_node->sigKeyStrokesListChanged();
         m_node->setDirty();
     }
 
@@ -914,7 +916,7 @@ struct SetKeyStrokeColorsCommand : public KUndo2Command {
         *m_list = m_oldList;
 
         m_node->setNeedsUpdate(true);
-        emit m_node->sigKeyStrokesListChanged();
+        Q_EMIT m_node->sigKeyStrokesListChanged();
         m_node->setDirty();
     }
 
