@@ -40,6 +40,7 @@
 #include <KisStorageModel.h>
 #include <KoMD5Generator.h>
 #include <KoResourceLoadResult.h>
+#include <KisResourceThumbnailCache.h>
 
 #include "ResourceDebug.h"
 
@@ -50,7 +51,6 @@ public:
     QString resourceLocation;
     QMap<QString, KisResourceStorageSP> storages;
     QHash<QPair<QString, QString>, KoResourceSP> resourceCache;
-    QMap<QPair<QString, QString>, QImage> thumbnailCache;
     QMap<QPair<QString, QString>, KisTagSP> tagCache;
     QStringList errorMessages;
 };
@@ -151,24 +151,6 @@ bool KisResourceLocator::resourceCached(QString storageLocation, const QString &
     return d->resourceCache.contains(key);
 }
 
-void KisResourceLocator::cacheThumbnail(QString storageLocation, const QString &resourceType, const QString &filename,
-                                        const QImage &img) {
-    storageLocation = makeStorageLocationAbsolute(storageLocation);
-    QPair<QString, QString> key = QPair<QString, QString> (storageLocation, resourceType + "/" + filename);
-
-    d->thumbnailCache[key] = img;
-}
-
-QImage KisResourceLocator::thumbnailCached(QString storageLocation, const QString &resourceType, const QString &filename)
-{
-    storageLocation = makeStorageLocationAbsolute(storageLocation);
-    QPair<QString, QString> key = QPair<QString, QString> (storageLocation, resourceType + "/" + filename);
-    if (d->thumbnailCache.contains(key)) {
-        return d->thumbnailCache[key];
-    }
-    return QImage();
-}
-
 void KisResourceLocator::loadRequiredResources(KoResourceSP resource)
 {
     QList<KoResourceLoadResult> requiredResources = resource->requiredResources(KisGlobalResourcesInterface::instance());
@@ -425,8 +407,8 @@ bool KisResourceLocator::setResourceActive(int resourceId, bool active)
     QPair<QString, QString> key = QPair<QString, QString> (rs.storageLocation, rs.resourceType + "/" + rs.resourceFileName);
 
     d->resourceCache.remove(key);
-    if (!active && d->thumbnailCache.contains(key)) {
-        d->thumbnailCache.remove(key);
+    if (!active) {
+        KisResourceThumbnailCache::instance()->remove(key);
     }
 
     bool result = KisResourceCacheDb::setResourceActive(resourceId, active);
@@ -592,7 +574,7 @@ KoResourceSP KisResourceLocator::importResource(const QString &resourceType, con
         const QPair<QString, QString> key = {absoluteStorageLocation, resourceType + "/" + resource->filename()};
         // Add to the cache
         d->resourceCache[key] = resource;
-        d->thumbnailCache[key] = resource->thumbnail();
+        KisResourceThumbnailCache::instance()->insert(key, resource->thumbnail());
 
         return resource;
     }
@@ -676,7 +658,7 @@ bool KisResourceLocator::updateResource(const QString &resourceType, const KoRes
     if (!storage->supportsVersioning()) return false;
 
     // remove older version
-    d->thumbnailCache.remove(QPair<QString, QString> (storageLocation, resourceType + "/" + resource->filename()));
+    KisResourceThumbnailCache::instance()->remove(storageLocation, resourceType, resource->filename());
 
     resource->updateThumbnail();
     resource->setVersion(resource->version() + 1);
@@ -705,7 +687,7 @@ bool KisResourceLocator::updateResource(const QString &resourceType, const KoRes
     // Update the resource in the cache
     QPair<QString, QString> key = QPair<QString, QString> (storageLocation, resourceType + "/" + resource->filename());
     d->resourceCache[key] = resource;
-    d->thumbnailCache[key] = resource->thumbnail();
+    KisResourceThumbnailCache::instance()->insert(key, resource->thumbnail());
 
     return true;
 }
@@ -778,8 +760,8 @@ void KisResourceLocator::purge(const QString &storageLocation)
 {
     Q_FOREACH(const auto key, d->resourceCache.keys()) {
         if (key.first == storageLocation) {
-            d->resourceCache.take(key);
-            d->thumbnailCache.take(key);
+            d->resourceCache.remove(key);
+            KisResourceThumbnailCache::instance()->remove(key);
         }
     }
 }
@@ -928,7 +910,7 @@ void KisResourceLocator::saveTags()
         QFile f(resourceLocation + "/" + tag->resourceType() + '/' + filename);
 
         if (!f.open(QFile::WriteOnly)) {
-            qWarning () << "Couild not open tag file for writing" << f.fileName();
+            qWarning () << "Could not open tag file for writing" << f.fileName();
             continue;
         }
 
@@ -970,7 +952,7 @@ QString KisResourceLocator::filePathForResource(KoResourceSP resource)
 
 KisResourceLocator::LocatorError KisResourceLocator::firstTimeInstallation(InitializationStatus initializationStatus, const QString &installationResourcesLocation)
 {
-    emit progressMessage(i18n("Krita is running for the first time. Initialization will take some time."));
+    Q_EMIT progressMessage(i18n("Krita is running for the first time. Initialization will take some time."));
     Q_UNUSED(initializationStatus);
 
     Q_FOREACH(const QString &folder, KisResourceLoaderRegistry::instance()->resourceTypes()) {
@@ -1002,7 +984,7 @@ KisResourceLocator::LocatorError KisResourceLocator::firstTimeInstallation(Initi
     QDirIterator iter(installationResourcesLocation, filters, QDir::Files, QDirIterator::Subdirectories);
     while (iter.hasNext()) {
         iter.next();
-        emit progressMessage(i18n("Installing the resources from bundle %1.", iter.filePath()));
+        Q_EMIT progressMessage(i18n("Installing the resources from bundle %1.", iter.filePath()));
         QFile f(iter.filePath());
         Q_ASSERT(f.exists());
         if (!f.copy(d->resourceLocation + '/' + iter.fileName())) {
@@ -1024,7 +1006,7 @@ KisResourceLocator::LocatorError KisResourceLocator::firstTimeInstallation(Initi
 
 bool KisResourceLocator::initializeDb()
 {
-    emit progressMessage(i18n("Initializing the resources."));
+    Q_EMIT progressMessage(i18n("Initializing the resources."));
     d->errorMessages.clear();
     findStorages();
 
@@ -1209,7 +1191,7 @@ bool KisResourceLocator::synchronizeDb()
                 qWarning() << d->errorMessages;
                 return false;
             }
-            emit storageRemoved(this->makeStorageLocationAbsolute(location));
+            Q_EMIT storageRemoved(this->makeStorageLocationAbsolute(location));
         }
     }
 
