@@ -27,7 +27,11 @@
 #include "kis_dummies_facade_base.h"
 #include "kis_node_dummies_graph.h"
 #include "KisImportExportManager.h"
-#include "KisImageBarrierLockerWithFeedback.h"
+#include "KisImageBarrierLock.h"
+#include <commands/kis_image_layer_add_command.h>
+#include <commands/kis_image_layer_move_command.h>
+#include <kis_processing_applicator.h>
+
 
 #include <KoProperties.h>
 #include <KoStore.h>
@@ -72,7 +76,11 @@ void KisMimeData::deepCopyNodes()
     KisNodeList newNodes;
 
     {
-        KisImageBarrierLockerWithFeedbackAllowNull locker(m_image);
+        KisImageReadOnlyBarrierLock lock(m_image, std::defer_lock);
+        if (m_image) {
+            lock.lock();
+        }
+
         Q_FOREACH (KisNodeSP node, m_nodes) {
             KisNodeSP newNode = safeCopyNode(node);
             newNode->setImage(nullptr);
@@ -121,11 +129,15 @@ KisDocument *createDocument(QList<KisNodeSP> nodes, KisImageSP srcImage, const Q
     }
 
     {
-        KisImageBarrierLockerWithFeedbackAllowNull locker(srcImage);
+        KisImageReadOnlyBarrierLock lock(srcImage, std::defer_lock);
+        if (srcImage) {
+            lock.lock();
+        }
+
         Q_FOREACH (KisNodeSP node, nodes) {
             KisNodeSP clonedNode = safeCopyNode(node);
-            /// HACK ALERT: here we just initilize parent image link
-            ///             and skip initilizing shapeController!
+            /// HACK ALERT: here we just initialize parent image link
+            ///             and skip initializing shapeController!
             ///             Ideally, we should call initializeExternalNode()
             ///             instead.
             image->addNode(clonedNode);
@@ -171,7 +183,7 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
 
         QScopedPointer<KisDocument> doc(createDocument(m_nodes, m_image, m_copiedBounds));
 
-        return doc->image()->projection()->convertToQImage(cfg.displayProfile(QApplication::desktop()->screenNumber(qApp->activeWindow())),
+        return doc->image()->projection()->convertToQImage(cfg.displayProfile(QApplication::desktop()->screenNumber(QApplication::activeWindow())),
                                                            KoColorConversionTransformation::internalRenderingIntent(),
                                                            KoColorConversionTransformation::internalConversionFlags());
     }
@@ -252,30 +264,28 @@ QList<KisNodeSP> KisMimeData::tryLoadInternalNodes(const QMimeData *data,
         QDomDocument doc;
         doc.setContent(nodeXml);
 
-        QDomElement element = doc.documentElement();
-        qint64 pid = element.attribute("application_pid").toLongLong();
-        forceCopy = element.attribute("force_copy").toInt();
-        qint64 imagePointerValue = element.attribute("image_pointer_value").toLongLong();
+        QDomElement root = doc.documentElement();
+        qint64 pid = root.attribute("application_pid").toLongLong();
+        forceCopy = root.attribute("force_copy").toInt();
+        qint64 imagePointerValue = root.attribute("image_pointer_value").toLongLong();
         sourceImage = reinterpret_cast<KisImage*>(imagePointerValue);
 
         if (pid == QApplication::applicationPid()) {
-
-            QDomNode n = element.firstChild();
-            while (!n.isNull()) {
-                QDomElement e = n.toElement();
-                if (!e.isNull()) {
-                    qint64 pointerValue = e.attribute("pointer_value").toLongLong();
-                    if (pointerValue) {
-                        nodes << reinterpret_cast<KisNode*>(pointerValue);
-                    }
+            QDomElement e;
+            for (e = root.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
+                qint64 pointerValue = e.attribute("pointer_value").toLongLong();
+                if (pointerValue) {
+                    nodes << reinterpret_cast<KisNode*>(pointerValue);
                 }
-                n = n.nextSibling();
             }
         }
     }
 
     if (!nodes.isEmpty() && (forceCopy || copyNode || sourceImage != image)) {
-        KisImageBarrierLockerWithFeedbackAllowNull locker(sourceImage);
+        KisImageReadOnlyBarrierLock lock(sourceImage, std::defer_lock);
+        if (sourceImage) {
+            lock.lock();
+        }
 
         QList<KisNodeSP> clones;
         Q_FOREACH (KisNodeSP node, nodes) {
@@ -293,8 +303,7 @@ QList<KisNodeSP> KisMimeData::tryLoadInternalNodes(const QMimeData *data,
 }
 
 QList<KisNodeSP> KisMimeData::loadNonNativeNodes(const QMimeData *data,
-                                                KisImageWSP image,
-                                                KisShapeController *shapeController)
+                                                 KisImageWSP image)
 {
     bool doRecenter = false;
     QList<KisNodeSP> nodes;
@@ -351,7 +360,7 @@ QMimeData* KisMimeData::mimeForLayers(const KisNodeList &nodes, KisImageSP image
 {
     KisNodeList inputNodes = nodes;
     KisNodeList sortedNodes;
-    KisLayerUtils::sortMergableNodes(image->root(), inputNodes, sortedNodes);
+    KisLayerUtils::sortMergeableNodes(image->root(), inputNodes, sortedNodes);
     if (sortedNodes.isEmpty()) return 0;
 
     KisMimeData* data = new KisMimeData(sortedNodes, image, forceCopy);
@@ -362,7 +371,7 @@ QMimeData* KisMimeData::mimeForLayersDeepCopy(const KisNodeList &nodes, KisImage
 {
     KisNodeList inputNodes = nodes;
     KisNodeList sortedNodes;
-    KisLayerUtils::sortMergableNodes(image->root(), inputNodes, sortedNodes);
+    KisLayerUtils::sortMergeableNodes(image->root(), inputNodes, sortedNodes);
     if (sortedNodes.isEmpty()) return 0;
 
     KisMimeData* data = new KisMimeData(sortedNodes, image, forceCopy);
@@ -386,6 +395,8 @@ bool correctNewNodeLocation(KisNodeList nodes,
                             KisNodeDummy* &parentDummy,
                             KisNodeDummy* &aboveThisDummy)
 {
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(parentDummy, false);
+
     KisNodeSP parentNode = parentDummy->node();
     bool result = true;
 
@@ -414,7 +425,7 @@ KisNodeList KisMimeData::loadNodesFast(
                                           copyNode /* IN-OUT */);
 
     if (nodes.isEmpty()) {
-        nodes = KisMimeData::loadNonNativeNodes(data, image, shapeController);
+        nodes = KisMimeData::loadNonNativeNodes(data, image);
         /**
          * Don't try to move a node originating from another image,
          * just copy it.
@@ -448,7 +459,8 @@ bool KisMimeData::insertMimeLayers(const QMimeData *data,
                                    bool copyNode,
                                    KisNodeInsertionAdapter *nodeInsertionAdapter,
                                    bool changeOffset,
-                                   QPointF offset)
+                                   QPointF offset,
+                                   KisProcessingApplicator *applicator)
 {
     QList<KisNodeSP> nodes = loadNodesFast(data, image, shapeController, copyNode /* IN-OUT */);
 
@@ -471,16 +483,39 @@ bool KisMimeData::insertMimeLayers(const QMimeData *data,
         return false;
     }
 
-    KIS_ASSERT_RECOVER(nodeInsertionAdapter) { return false; }
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(nodeInsertionAdapter, false);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(parentDummy, false);
 
-    Q_ASSERT(parentDummy);
     KisNodeSP aboveThisNode = aboveThisDummy ? aboveThisDummy->node() : 0;
-    if (copyNode) {
-        nodeInsertionAdapter->addNodes(nodes, parentDummy->node(), aboveThisNode);
-    }
-    else {
-        Q_ASSERT(nodes.first()->graphListener() == image.data());
-        nodeInsertionAdapter->moveNodes(nodes, parentDummy->node(), aboveThisNode);
+
+    if (!applicator) {
+        if (copyNode) {
+            nodeInsertionAdapter->addNodes(nodes, parentDummy->node(), aboveThisNode);
+        }
+        else {
+            KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(nodes.first()->graphListener() == image.data(), false);
+            nodeInsertionAdapter->moveNodes(nodes, parentDummy->node(), aboveThisNode);
+        }
+    } else {
+        {
+            if (copyNode) {
+                KisNodeSP prevNode = aboveThisNode;
+                Q_FOREACH (KisNodeSP node, nodes) {
+                    applicator->applyCommand(
+                        new KisImageLayerAddCommand(image, node, parentDummy->node(), prevNode));
+                    prevNode = node;
+                }
+            } else {
+                KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(nodes.first()->graphListener() == image.data(), false);
+
+                KisNodeSP prevNode = aboveThisNode;
+                Q_FOREACH (KisNodeSP node, nodes) {
+                    applicator->applyCommand(
+                        new KisImageLayerMoveCommand(image, node, parentDummy->node(), prevNode));
+                    prevNode = node;
+                }
+            }
+        }
     }
 
     const bool hasDelayedNodes =
