@@ -307,7 +307,7 @@ void KisSelectionManager::updateGUI()
 //    m_save->setEnabled(havePixelsSelected);
 
     updateStatusBar();
-    emit signalUpdateGUI();
+    Q_EMIT signalUpdateGUI();
 }
 
 void KisSelectionManager::updateStatusBar()
@@ -320,7 +320,7 @@ void KisSelectionManager::updateStatusBar()
 void KisSelectionManager::selectionChanged()
 {
     m_view->updateGUI();
-    emit currentSelectionChanged();
+    Q_EMIT currentSelectionChanged();
 }
 
 void KisSelectionManager::cut()
@@ -427,8 +427,8 @@ void KisSelectionManager::editSelection()
 
     if (!action->isChecked()) {
         action->setChecked(true);
-        emit action->toggled(true);
-        emit action->triggered(true);
+        Q_EMIT action->toggled(true);
+        Q_EMIT action->triggered(true);
     }
 
     KisNodeSP node = selection->parentNode();
@@ -544,7 +544,7 @@ void KisSelectionManager::toggleDisplaySelection()
     m_toggleDisplaySelection->setChecked(m_selectionDecoration->visible());
     m_toggleDisplaySelection->blockSignals(false);
 
-    emit displaySelectionChanged();
+    Q_EMIT displaySelectionChanged();
 }
 
 bool KisSelectionManager::displaySelection()
@@ -618,7 +618,7 @@ void KisSelectionManager::slotToggleSelectionDecoration()
         KisSelectionDecoration::Ants : KisSelectionDecoration::Mask;
 
     m_selectionDecoration->setMode(mode);
-    emit displaySelectionChanged();
+    Q_EMIT displaySelectionChanged();
 }
 
 bool KisSelectionManager::showSelectionAsMask() const
